﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;
using HeuristicLab.Problems.Instances;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment.Views {
  [View("GeneralizedQuadraticAssignmentProblemView")]
  [Content(typeof(IProblemInstanceProvider), IsDefaultView = true)]
  public partial class ProblemInstanceProviderView : AsynchronousContentView {

    public new IProblemInstanceProvider Content {
      get { return (IProblemInstanceProvider)base.Content; }
      set { base.Content = value; }
    }

    public ProblemInstanceProviderView() {
      InitializeComponent();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        instancesComboBox.DataSource = null;
      } else {
        instancesComboBox.DisplayMember = "Name";
        instancesComboBox.DataSource = Content.GetInstanceDescriptors().ToList();
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      instancesComboBox.Enabled = !ReadOnly && !Locked && Content != null && Content.ConsumerCanBeFed;
      loadButton.Enabled = !ReadOnly && !Locked && Content != null && Content.ConsumerCanBeFed;
    }

    private void loadButton_Click(object sender, EventArgs e) {
      var instance = (IInstanceDescriptor)instancesComboBox.SelectedItem;
      Content.FeedConsumer(instance);
    }

    private void comboBox_DataSourceChanged(object sender, EventArgs e) {
      var comboBox = (ComboBox)sender;
      if (comboBox.DataSource == null)
        comboBox.Items.Clear();
    }
  }
}
